HashResult

Struct HashResult 

#[repr(C, packed(1))]
pub struct HashResult {
Show 13 fields pub result: i32, pub hash_format: i32, pub header_dimensions_image_x: i32, pub header_dimensions_image_y: i32, pub header_dimensions_image_w: i32, pub header_dimensions_image_h: i32, pub hash: [u8; 1232], pub reserved0: i32, pub reserved1: i32, pub reserved2: i32, pub reserved3: i32, pub reserved4: i32, pub reserved5: i32,
}
Result structure returned by border detection hash functions.

When a border is found, the hash with the border removed will be in the second instance of the results array.

Result Values

  • < 0: An error occurred (see error codes)
  • 1: No border was found
  • 2: A border was found

Fields

result: i32

Error code if less than 0, otherwise indicates border detection result.

hash_format: i32

Hash format used for this result.

header_dimensions_image_x: i32

Left position (X) within the provided image.

header_dimensions_image_y: i32

Top position (Y) within the provided image.

header_dimensions_image_w: i32

Width within the provided image.

header_dimensions_image_h: i32

Height within the provided image.

hash: [u8; 1232]

The computed hash in the requested format.

reserved0: i32

Reserved for future use.

reserved1: i32

Reserved for future use.

reserved2: i32

Reserved for future use.

reserved3: i32

Reserved for future use.

reserved4: i32

Reserved for future use.

reserved5: i32

Reserved for future use.